/*---------------------------------------------------------------------------+
|       Program realizujcy operacj rysowania acucha liniowego            |
|            Krzysztof Binia, mail: skorpion@opgk.krakow.pl                 |
+---------------------------------------------------------------------------*/

/*Pliki nagwkowe*/
#include <mselems.h>
#include <global.h>
#include <mdl.h>
#include <tcb.h>
#include <userfnc.h>
#include <cmdlist.h>

/*Pliki deklaracji funkcji*/
#include <mselemen.fdf>
#include <msstate.fdf>
#include <msoutput.fdf>

#define  MAX_VERTEXES  101

Private Dpoint3d vertexes[MAX_VERTEXES]; /*Tablica wierzchokw*/
Private int      index;                  /*Indeks biecego wierzchoka*/

/*---------------------------------------------------------------------------+
|    addElement                                                              |
+---------------------------------------------------------------------------*/
/*Funkcja dodajca (w zalenoci od liczby punktw w tablicy wierzchokw)*/
/*element liniowy lub acuch liniowy do pliku*/
Private void addElement (void)
{
 MSElementUnion el;

 if( index>1 ) /*Utworzenie acucha liniowego*/
  mdlLineString_create(&el,NULL,vertexes,index+1);
 else /*Utworzenie linii*/
  mdlLine_create(&el,NULL,vertexes);

 /*Wywietlenie elementu na ekranie*/
 mdlElement_display(&el,NORMALDRAW);
 /*Zapis elementu do pliku*/
 mdlElement_add(&el);
}

/*---------------------------------------------------------------------------+
|    restartCommand                                                          |
+---------------------------------------------------------------------------*/
/*Wstawienie elementu oraz restart komendy "ls" - w wyniku wcinicia*/
/*przez uytkownika prawego przycisku myszy*/
Private void restartCommand(void)
{
 /*Jeeli w tablicy wierzchokw s conajmniej 2 punkty to zapisz element*/
 if(index>=1)
 {
  addElement();
  index=1;
 }
 /*Restart biecej komendy*/
 mdlState_restartCurrentCommand();
}

/*---------------------------------------------------------------------------+
|    createLine                                                              |
+---------------------------------------------------------------------------*/
/*Funkcja generujca w buforze Microstation tymczasowy element liniowy*/
Private int createLine (DPoint3d *pointP, int view)
{
 /*Dodanie punktu do tabeli wierzchokw*/
 vertexes[index+1]=*pointP;
 /*Utworzenie linii w buforze Microstation (dgnBuf)*/
 mdlLine_create(dgnBuf,NULL,&vertexes[index]);

 return SUCCESS;
}

/*---------------------------------------------------------------------------+
|    nextPoint                                                               |
+---------------------------------------------------------------------------*/
/*Funkcja pobierajca kolejno wskazane przez operatora punkty*/
Private void nextPoint (Dpoint3d *pointP,int view)
{
 char      status[20];
 MSElement el;

 index++;
 /*Dodanie punktu do tabeli wierzchokw*/
 vertexes[index]=*pointP;
 /*Utworzenie (wypenienie struktury el) elementu liniowego*/
 mdlLine_create(&el,NULL,&vertexes[index-1]);
 /*Wywietlenie linii na ekranie monitora,*/
 /*linia nie jest jeszcze zapisywana w rysunku, jest tylko wywietlana !!!*/
 mdlElement_display(&el,NORMALDRAW);

 /*Jesli liczba wskazanych punktw jest rwna MAX_VERTEXES nastpuje*/
 /*zapis stworzonego acucha liniowego do pliku - funkcj addElement()*/
 if(index==(MAX_VERTEXES-1))
 {
  addElement();
  vertexes[0]=*pointP;
  index=0;                  
 }
       
 mdlOutput_printf (MSG_ERROR,"Line string > Wska nastpny punkt");
 sprintf (status,"Wierzchoek: %d\0",index+1);
 mdlOutput_printf (MSG_STATUS,status);
}

/*---------------------------------------------------------------------------+
|    firstPoint                                                              |
+---------------------------------------------------------------------------*/
/*Funkcja wstawiajca pierwszy wskazany punkt do tablicy wierzchokw*/
Private void firstPoint (Dpoint3d *pointP, int view)
{
 char status[20];

 /*Dodanie punktu do tablicy wierzchokw*/
 vertexes[0]=*pointP;

 /*Pod lewy przycisk myszy "podpinamy" teraz funkcj nextPoint*/
 mdlState_setFunction(STATE_DATAPOINT,nextPoint);
 /*A pod prawy restartCommand*/
 mdlState_setFunction(STATE_RESET,restartCommand);
 /*Wczenie obsugi ruchu myszy na ekranie monitora, podczas zmiany*/
 /*pooenia kursora nastpuj kolejne wywoania funkcji createLine,*/
 /*tworzcej tymczasow lini w buforze Microstation*/
 mdlState_dynamicUpdate(createLine,FALSE);

 mdlOutput_printf(MSG_ERROR,"Line string > Wska nastpny punkt");
 sprintf(status,"Wierzchoek: %d\0",index+1);
 mdlOutput_printf(MSG_STATUS,status);
}

/*---------------------------------------------------------------------------+
|    ls                                                                      |
+---------------------------------------------------------------------------*/
/*Wywoanie komendy programu - key-in:"mdl command ls"*/
cmdName void ls(void)
{
 index=0;
 /*Funkcja startujca komend klasy Primitive, tworzc elementy w rysunku*/
 /*Pierwszy parametr - funkcja firstPoint() odpowiedzialny jest za obsug*/
 /*lewego przycisku myszy, drugi parametr ls() za obsug przycisku prawego*/
 mdlState_startPrimitive(firstPoint,ls,0,0);
 mdlOutput_printf(MSG_ERROR,"Line string > Wska pierwszy punkt");
}

/*---------------------------------------------------------------------------+
|    main                                                                    |
+---------------------------------------------------------------------------*/
/*Gwna funkcja programu wywoywana w momencie adowania programu do pamici*/
void main (int argc,char *argv[])
{
}
